from tornado.util import import_object
from settings import settings

class AdminSite(object):

    AVAILABLE_COLLECTIONS = {}

    def register(self, cls):
        #print 'reging', cls
        self.AVAILABLE_COLLECTIONS[cls.__name__.lower()] = cls

    def autodiscover(self):        
        for app in settings.get('apps', []):
            try:
                __import__(app, None, None, ['admin']).admin
            except:

                print app, 'failed'

    def get_class_for_name(self, name):
        return self.AVAILABLE_COLLECTIONS[name]

    def get_collections(self):
        return self.AVAILABLE_COLLECTIONS

admin = AdminSite()



#coding=utf-8
import re
import cgi
from ramen.models import ActiveRecord, ActiveRecordFactory
from ramen.rules import Rules

from settings import DB

class Form(object):

    def __init__(self, form_model, *args, **kwargs):
        self.form_model = form_model
        super(Form, self).__init__(*args, **kwargs)

    def _default_html_data(self):
        return {
            'tag': 'input',
            'class': '',
            'type': 'text',
            'value': '',
            'name': '',
            'name_suffix': '',
            'extra': '',
            'hint': '',
            'label': ''
        }

    def _create_html_tag(self, data={}, single=True):

        params = self._default_html_data()
        params.update(data)


        if hasattr(self.form_model, '_errors'):
            if data.get('name') in self.form_model._errors:
                params['class'] += ' error'
                params['hint'] = u'<br/>'.join(self.form_model._errors[data['name']])

        if single:
            return '<p class="%(class)s"><label>%(label)s<%(tag)s name="%(name)s%(name_suffix)s" type="%(type)s" value="%(value)s" %(extra)s/></label><span>%(hint)s</span></p>\n' % params
        else:
            return '<p class="%(class)s"><label>%(label)s<%(tag)s name="%(name)s%(name_suffix)s" %(extra)s>%(value)s</%(tag)s></label><span>%(hint)s</span></p>\n' % params

    def _guess_method(self, rules, value, default=None):
        tag = 'input'
        data = {}
        rules = rules[:]
        for rule in rules:
            if rule.name == 'fk':
                rule.setobject(self.form_model)
                data['collection'] = rule.ar.objects.all()
                data['collection'].load()
                data['display'] = 'admin_title'
                #if len(data['collection']) > 5:
                tag = 'select'
                if type(value) in [list, tuple]:
                    data['extra'] = 'multiple'

                #else:
                #    if type(value) in [list, tuple]:
                #        tag = 'checkgroup'
                #    else:
                #        tag = 'radiogroup'

                print 'ASDASD', data['collection']
                if not 'required' in [x.name for x in rules]: # and tag == 'select':
                    data['collection'].insert(0, {'_id':0, data['display']:'---'})

                print 'ASDASD', data['collection']

                if not (('required' in [x.name for x in rules]) or len(data['collection'])):
                    tag = 'input'
                    data = {}
                    data['type'] = 'hidden'
                    data['value'] = default

                rules.remove(rule)
            if rule.name == 'unique':
                rules.remove(rule)
        print tag, data, rules
        return tag, data, rules

    def render(self, exclude = []):
        s = []
        print 'RENDER'
        for r in self.form_model._rules:
            name, rules, d = (r + ('',))[:3]
            if name in exclude:
                continue
            tag_method, kwargs, rules = self._guess_method(rules, self.form_model.get(name, ''), d)
            if rules:
                kwargs['extra'] = cgi.escape('%s%s' % (kwargs.get('extra', ''), ' accept="%s"' % '|'.join([r.name for r in rules])))
            method = self.__getattribute__(tag_method)
            if isinstance(d, list) and tag_method in ['input', 'textarea']:
                vals = self.form_model.get(name, []) or d
                print 'V', vals
                for v in vals:
                    kwargs['value'] = v
                    kwargs['label'] = name
                    s.append(method(name = '%s[]' % name, **kwargs))
            elif isinstance(d, dict) and tag_method in ['input', 'textarea']:
                vals = self.form_model.get(name, {}) or d
                for k,v in vals.iteritems():
                    kwargs['value'] = v
                    kwargs['label'] = name
                    s.append(method(name = '%s[%s]' % (name, k), **kwargs))
            else:
                s.append(method(name = name, **kwargs))
        return ''.join(s)

    def input(self, name, type='text', hint='', extra='', value=None, label=''):
        return self._create_html_tag({
            'tag': 'input',
            'name': name,
            'type': type,
            'value': (value is not None) and str(value) or self.form_model.get(name, ''),
            'extra': extra,
            'hint': hint,
            'label': type == 'hidden' and ' ' or (label or name)
        })

    def textarea(self, name, hint='', extra='', value = None, label=''):
        return self._create_html_tag({
            'tag': 'textarea',
            'name': name,
            'value': (value is not None) and str(value) or self.form_model.get(name, ''),
            'extra': extra,
            'hint': hint,
            'label': label or name
        }, single=False)

    def select(self, name, options = [], collection = None, value='_id', display='_id', hint='', extra=''):
        if collection:
            options = [(obj[value], obj.get(display, ':(')) for obj in collection]
        _tmp = []
        multi = type(self.form_model.get(name, '')) in [list, tuple]
        for option in options:
            value, display = option
            if multi:
                selected = value in self.form_model.get(name, '') and 'selected' or ''
            else:
                selected = value == self.form_model.get(name, '') and 'selected' or ''
            _tmp.append('<option %s value="%s">%s</option>' % (selected, value, display))

        return self._create_html_tag({
            'tag': 'select',
            'name': name,
            'name_suffix': multi and '[]' or '',
            'extra': multi and 'multiple' or '',
            'value': ''.join(_tmp),
            'extra': extra,
            'hint': hint
        }, single=False)

    def radiogroup(self, name, options = [], collection = None, value='_id', display='_id', hint='', extra=''):
        print collection
        if collection:
            options = [(obj[value], obj.get(display, ':(')) for obj in collection]
        _tmp = []
        for option in options:
            value, display = option
            _tmp.append('<label><input type="radio" name="%s" value="%s" %s>%s</label>' % (name, value, value == self.form_model.get(name, '') and 'checked' or '', display, ))

        return self._create_html_tag({
            'tag': 'ul',
            'value': ''.join(_tmp),
            'extra': extra,
            'hint': hint
        }, single=False)

    def checkgroup(self, name, options = [], collection = None, value='_id', display='_id', hint='', extra=''):
        if collection:
            options = [(obj[value], obj[display]) for obj in collection]
        _tmp = []
        for option in options:
            value, display = option
            checked = (type(self.form_model.get(name, 0)) in [list, tuple] and value in self.form_model.get(name, [])) and 'checked' or ''
            _tmp.append('<label><input type="checkbox" name="%s[]" value="%s" %s>%s</label>' % (name, value, checked, display, ))

        return self._create_html_tag({
            'tag': 'ul',
            'value': ''.join(_tmp),
            'extra': extra,
            'hint': hint
        }, single=False)

    def submit(self, value='Submit', extra=''):
        return self._create_html_tag({
            'tag': 'input',
            'type': 'submit',
            'extra': extra,
            'value': value
        })

import sys
class FormModel(Rules):

    def __init__(self, *args, **kwargs):
        super(FormModel, self).__init__(*args, **kwargs)

    def is_valid(self, arguments):
        pod = lambda x : not isinstance(x, type([])) and not isinstance(x, type({}))
        self._errors = {}
        bad = 0
        for r in self._rules:
            k, rules, d = (r + ('',))[:3]

            try:
                value = arguments.get(k, False)
                if value == False:
                    continue

                if not value:
                    value = d

                err = self.validate(k, rules, value if pod(value) else value)
                if len(err) > 0:
                    bad += 1
                    self._errors[k] = err
            except Exception as e:
                print 'for key:', k, " Unexpected error:", sys.exc_info()[0], e
                raise

        #print self._errors
        return bad == 0

    def validate(self, k, rules, value):

        errors = []
        old_value = value

        for r in rules:
            r.setvalue(value)
            r.setobject(self)

            if not r():
                errors.extend(r.errors)

            value = r.val

        if len(errors):
            self[k] = old_value
        else:
            self[k] = value

        return errors

class ValidationError(Exception):
    pass


#class Security(object):
#    """Security - input data check"""
#    def __init__(self, rules, value):
#        self.rules = rules
#        self.value = value
#        self.errors = []
#
#    @property
#    def val(self):
#        return self.value
#
#    def perform(self):
#        """Выполнить проверку данных"""
#        regex = re.compile(r"^(fk|length|integer|double|unique)(\<|\>|\=|\>=|\<=)([a-z\.\_0-9\[\]\-]+)$")
#        bad = 0
#        for rule in self.rules:
#            if rule in ['xss_strip','email','integer','numeric','tag_strip','required','trim','double','password','coords']:
#                result = self.__getattribute__(rule)()
#            else:
#                m = regex.match(rule)
#                if m != None:
#                    result = self.__getattribute__(m.group(1))(op=m.group(2), arg=m.group(3))
#                else:
#                    raise ValidationError('Rule %s not found. May be in the future :)' % rule)
#
#            if not result:
#                bad += 1
#
#        return bad == 0
#
#    def xss_strip(self, **kwargs):
#        """вырезать небезопасные вставки HTML"""
#        regex_expr = r'<(script|object|embed|iframe|applet|body|frameset|frame|head|html|form|vbscript)[^>]*?>(.*?)</\1>'
#        if isinstance(self.value, type([])):
#            new_value = []
#            for v in self.value:
#                new_value.append(re.sub(regex_expr, '', v, re.I | re.M | re.S))
#            self.value = new_value
#        elif isinstance(self.value, type({})):
#            new_value = {}
#            for k, v in self.value.iteritems():
#                new_value[k] = re.sub(regex_expr, '', v, re.I | re.M | re.S)
#            self.value = new_value
#        else:
#            self.value = re.sub(regex_expr, '', self.value, re.I | re.M | re.S)
#
#        return True
#
#    def tag_strip(self, **kwargs):
#        """удалить теги"""
#        regex_expr = '(\>|\<\/?)+'
#        if isinstance(self.value, type([])):
#            new_value = []
#            for v in self.value:
#                new_value.append(re.sub(regex_expr, '', v, re.I | re.M))
#            self.value = new_value
#        elif isinstance(self.value, type({})):
#            new_value = {}
#            for k, v in self.value.iteritems():
#                new_value[k] = re.sub(regex_expr, '', v, re.I | re.M)
#            self.value = new_value
#        else:
#            self.value = re.sub(regex_expr, '', self.value, re.I | re.M)
#
#        return True
#
#    def email(self, **kwargs):
#        """проверить поле на соответствие почтовому ящику"""
#        regex = re.compile('^([a-z0-9\+_\-]+)(\.[a-z0-9\+_\-]+)*@([a-z0-9\-]+\.)+[a-z]{2,6}$', re.I | re.U)
#        result = self.regex_match(regex)
#
#        if not result:
#            self.format_error('email')
#
#        return result
#
#
#    def integer(self, op=None, arg=None):
#        """является ли поле целым числом с приведением"""
#        result = self.regex_match(re.compile('^[0-9]+$'))
#
#        if not op and not arg and not result:
#            self.format_error('integer')
#        elif op and arg and not result:
#            self.format_error('integer')
#        if op and arg and result:
#            bad = 0
#            if isinstance(self.value, type([])):
#                for v in self.value:
#                    if not self.relop(op, int(v), int(arg)):
#                        bad += 1
#                        self.format_error('integer_param', op, int(arg))
#            elif isinstance(self.value, type({})):
#                for k, v in self.value.iteritems():
#                    if not self.relop(op, int(v), int(arg)):
#                        bad += 1
#                        self.format_error('integer_param', op, int(arg))
#            else:
#                print op, int(self.value), arg
#                if not self.relop(op, int(self.value), int(arg)):
#                    bad += 1
#                    self.format_error('integer_param', op, int(arg))
#
#            result = bad == 0
#
#        if result:
#            if isinstance(self.value, type([])):
#                new_v = []
#                for v in self.value:
#                    new_v.append(int(v))
#                self.value = new_v
#            elif isinstance(self.value, type({})):
#                new_v = {}
#                for k, v in self.value.iteritems():
#                    new_v[k] = int(v)
#                self.value = new_v
#            else:
#                self.value = int(self.value)
#
#        return result
#
#    def numeric(self, **kwargs):
#        """проверить поле на принадлежность числу"""
#        result = self.regex_match(re.compile(r'^\d+|\d+\.\d+|\.\d+$'))
#
#        if not result:
#            self.format_error('numeric')
#
#        return result
#
#    def double(self, op=None, arg=None):
#        """число с плавающей точкой с приведением"""
#        result = self.regex_match(re.compile(r'^-?\d+|\d+\.\d+|\.\d+$'))
#
#        if not op and not arg and not result:
#            self.format_error('double')
#
#        if op and arg and result:
#            bad = 0
#            if isinstance(self.value, type([])):
#                for v in self.value:
#                    if not self.relop(op, float(v), float(arg)):
#                        bad += 1
#                        self.format_error('double_param', op, float(arg))
#            elif isinstance(self.value, type({})):
#                for k, v in self.value.iteritems():
#                    if not self.relop(op, float(v), float(arg)):
#                        bad += 1
#                        self.format_error('double_param', op, float(arg))
#            else:
#                if not self.relop(op, float(self.value), float(arg)):
#                    bad += 1
#                    self.format_error('double_param', op, float(arg))
#
#            result = bad == 0
#
#        if result:
#            if isinstance(self.value, type([])):
#                new_v = []
#                for v in self.value:
#                    new_v.append(float(v))
#                self.value = new_v
#            elif isinstance(self.value, type({})):
#                new_v = {}
#                for k, v in self.value.iteritems():
#                    new_v[k] = float(v)
#                self.value = new_v
#            else:
#                self.value = float(self.value)
#
#        return result
#
#
#    def fk(self, op, arg):
#        """вторичный ключ для коллекции"""
#        AR = ActiveRecordFactory(arg)
#        print AR
#        bad = 0
#        assert len(self.value)
#        print 'V', self.value, type(self.value)
#        if isinstance(self.value, type([])):
#            for v in self.value:
#                if int(v) != 0 and not AR.is_exists(v):
#                    bad += 1
#                    self.format_error('fk')
#        elif isinstance(self.value, type({})):
#            for k, v in self.value.iteritems():
#                if int(v) != 0 and not AR.is_exists(v):
#                    bad += 1
#                    self.format_error('fk')
#        else:
#            # ar = AR(self.value)
#            if int(self.value) != 0 and not AR.is_exists(self.value):
#                bad += 1
#                self.format_error('fk')
#
#        return bad == 0
#
#    def required(self, **kwargs):
#        """обязательно существование значения"""
#        l = len(self.value)
#
#        if l == 0:
#            self.format_error('required')
#
#        return l > 0
#
#    def length(self, op, arg):
#        bad = 0
#        if isinstance(self.value, type([])):
#            for v in self.value:
#                if not self.relop(op, len(v), int(arg)):
#                    bad += 1
#                    self.format_error('length', op, int(arg))
#        elif isinstance(self.value, type({})):
#            for k, v in self.value.iteritems():
#                if not self.relop(op, len(v), int(arg)):
#                    bad += 1
#                    self.format_error('length', op, int(arg))
#        else:
#            if not self.relop(op, len(self.value), int(arg)):
#                bad += 1
#                self.format_error('length', op, int(arg))
#
#        return bad == 0
#
#    def trim(self, **kwargs):
#        """удалить пробельные символы в начале и конце строки"""
#        if isinstance(self.value, type([])):
#            new_value = []
#            for v in self.value:
#                new_value.append(v.strip())
#            self.value = new_value
#        elif isinstance(self.value, type({})):
#            new_value = {}
#            for k, v in self.value.iteritems():
#                new_value[k] = v.strip()
#            self.value = new_value
#        else:
#            self.value = self.value.strip()
#
#        return True
#
#    def password(self):
#        """пароль, значением должен быть список с двумя элементами"""
#        if isinstance(self.value, type([])) and self.value.len() == 2:
#            if len(self.value[0]) < 6:
#                self.format_error('password_length')
#                return False
#
#            if self.value[0] != self.value[1]:
#                self.format_error('password_mismatch')
#                return False
#
#            self.value = self.value[1]
#
#            return True
#
#        return False
#
#    def coords(self):
#        """географические координаты"""
#        if isinstance(self.value, list) and len(self.value) == 2:
#            if not self.value[0] and not self.value[1]:
#                return True
#
#            val0 = float(self.value[0])
#            val1 = float(self.value[1])
#
#            if val0 <= 180.0 and val0 >= -180.0 and val1 <= 180.0 and val1 >= -180.0:
#                self.value[0] = val0
#                self.value[1] = val1
#                return True
#
#            self.format_error('coords_bad_range')
#
#        return False
#
#    def unique(self, op, arg):
#        """проверить уникальность значения для данного поля в коллекции"""
#        m = re.match('^([a-z0-9\_]+)\[([a-z0-9\_]+)\]$', arg)
#        if not m:
#            raise ValidationError('Invalid rule syntax for <unique=collection[field]>')
#
#        db = DB
#        coll = db._db[m.group(1)]
#        field = m.group(2)
#        found = False
#        if isinstance(self.value, type([])):
#            for v in self.value:
#                if coll.find_one({field:v}):
#                    found = True
#        elif isinstance(self.value, type({})):
#            for k, v in self.value.iteritems():
#                if coll.find_one({field:v}):
#                    found = True
#        else:
#            if coll.find_one({field:self.value}):
#                found = True
#
#        if found:
#            self.format_error('unique')
#
#        return not found
#
#    def relop(self, reloper, source, target):
#        if reloper == '<':
#            return source < target
#        elif reloper == '>':
#            return source > target
#        elif reloper == '=':
#            return source == target
#        elif reloper == '>=':
#            return source >= target
#        elif reloper == '<=':
#            return source <= target
#
#        return False
#
#    def regex_match(self, regex):
#        bad = 0
#        if isinstance(self.value, type([])):
#            for v in self.value:
#                if not regex.match(v):
#                    bad += 1
#        elif isinstance(self.value, type({})):
#            for k, v in self.value.iteritems():
#                if not regex.match(v):
#                    bad += 1
#        else:
#            if not regex.match(self.value):
#                bad += 1
#
#        return bad == 0
#
#    errors_messages = {
#        'fk':u'Элемент не найден',
#        'double' :u'Параметр не является числом',
#        'double_param':u'Значение должно быть %(relop)s %(arg)g',
#        'email':u'Адрес электронной почты не корректен',
#        'integer':u'Значение не является целым числом',
#        'integer_param':u'Значение должно быть %(relop)s %(arg)d',
#        'numeric':u'Значение не является числом',
#        'required':u'Поле обязательно для заполнения',
#        'password_length':u'Длина пароля короткая',
#        'password_mismatch':u'Пароли не совпали',
#        'length': u'Размер должен быть %(relop)s %(arg)d',
#        'unique': u'Значение не может быть использовано',
#        'coords_bad_range':u'Координаты заданы неверно'
#    }
#    def format_error(self, token, relop=None, arg=None):
#        self.errors.append(self.errors_messages[token] % {'relop':relop,'arg':arg})
#
#



#coding=utf-8

MM_DEFAULT_FROM = 'www-data <root@localhost>'
MM_DEFAULT_MAILER = 'RamenMTA'
MM_DEFAULT_CRLF = '\r\n'

from hashlib import md5
import time, random, base64


def chunk_split(source, size=76, separator = MM_DEFAULT_CRLF):
    return separator.join([source[i:i+size] for i in range(0, len(source), size)])

def encode_content(string):
    return chunk_split(base64.b64encode(string))

def gen_boundary_hash():
    return md5( '%d%f%d' % (random.randint(100000, 999999), time.time(), random.randint(0, 1000)) ).hexdigest()

def encode_header(string, charset = 'UTF-8'):
    return '=?' + charset + '?B?' + base64.b64encode(string) + '?='

class MimemailException(Exception):
    pass



from tornado.util import import_object


def pattern(*args, **kwargs):
    temp = []
    for arg in args:
        if type(arg) == type(()):
            temp.append(arg)
            
        if type(arg) == type([]):
            temp.extend(arg)
        
    return temp
    
def include(prefix, src, namespace = None, extra_params = {}):
    if type(src) == str:
        package = import_object(src)
        patterns = package.patterns
    else:
        patterns = list(src)
    
    temp = []
    for pattern in patterns:
        if len(pattern) == 2:
            path, handler = pattern 
            kwargs = {}            
        if len(pattern) == 3:
            path, handler, kwargs = pattern
        
        #print path, handler, kwargs, namespace

        if extra_params:
            kwargs.update(extra_params)

        if namespace:
            kwargs.setdefault('namespaces', []).insert(0, namespace)
        
        if prefix.startswith('^'):
            _path = path 
            _suffix = ''

            if path.endswith('$'):
                _path = path[:-1]
                _suffix = '$'
                        
            if not (_path.endswith('/?') or _path.endswith('/')):
                _path = '%s/?%s' % (_path, _suffix)
                path = _path

            elif _path.endswith('/') and not _path.endswith('/?'):
                _path = '%s?%s' % (_path, _suffix)
                path = _path

        nkw = dict(kwargs)
        nkw.update(extra_params)

        temp.append(('%s%s' % (prefix, path), handler, nkw))
        
    
    return temp
    
    



"""
MimeMail support
"""

from .mail import *


__all__ = [
    'Mimemail',
    'MimemailPart',
    'MimemailPartImage',
    'MimemailPartAttachment',
    'MimemailPartText',
    'MimemailPartHtml'
]




from settings import settings

import motor



class DBProxy(object):

    def __init__(self):
        self.db = motor.MotorClient(
            settings['db'].get('host','localhost'),
            settings['db'].get('port', 27017)).open_sync()[settings['db']['name']]


    def find(self, collection, *args, **kwargs):
        return motor.Op(self.db[collection].find(*args, **kwargs).to_list)

    def find_one(self, collection, *args, **kwargs):
        return motor.Op(self.db[collection].find_one, *args, **kwargs)






# $Id: trigger.py 204 2012-07-28 08:52:19Z megido $
__all__ = ['Trigger','TGOP_INSERT','TGOP_UPDATE','TGOP_DELETE','TGEV_AFTER','TGEV_BEFORE']

def allbases_object(obj):
    bases = []
    if obj.__name__ not in ['FormModel','dict','Rules','object','DB']:
        bases.append(obj.__name__)
        
    for o in obj.__bases__:
        if o.__name__  not in ['FormModel','dict','Rules','object','DB']:
            bases.extend(allbases_object(o))
            
    return bases

TGOP_INSERT = 'INSERT'
TGOP_UPDATE = 'UPDATE'
TGOP_DELETE = 'DELETE'
TGEV_AFTER  = 'AFTER'
TGEV_BEFORE = 'BEFORE'

triggers = {
    TGOP_INSERT: {TGEV_AFTER:{},TGEV_BEFORE:{}},
    TGOP_UPDATE: {TGEV_AFTER:{},TGEV_BEFORE:{}},
    TGOP_DELETE: {TGEV_AFTER:{},TGEV_BEFORE:{}}
}

class Trigger():
    @staticmethod
    def connect(sender, functor, tgop, tgev):
        can_attach = True
        for klass in allbases_object(sender):
            if triggers[tgop][tgev].has_key(klass):
                if functor in triggers[tgop][tgev][klass]:
                    can_attach = False
                    
        if can_attach:
            if not triggers[tgop][tgev].has_key(sender.__name__):
                triggers[tgop][tgev][sender.__name__] = []
            triggers[tgop][tgev][sender.__name__].append(functor)
            
    @staticmethod
    def on_trigger(ar, tgop, tgev):
        for klass in allbases_object(ar.__class__):
            if triggers[tgop][tgev].has_key(klass):
                for t in triggers[tgop][tgev][klass]:
                    t(ar, tgev == TGEV_BEFORE)



__all__ = [
    'PJS'
]


def escape_value(value):
    if type(value) in [str, unicode]:
        value = '"{0}"'.format(value)
    return value


class Attr(dict):
    
    def __str__(self):
        if self['namespace']:
            return u'{namespace}.{attr_name}'.format(**self)
        else:
            return self['attr_name']


class AttrAccessor(object):

    def __init__(self, attr_name):
        self.attr_name = attr_name

    def __get__(self, obj, owner):
        attr = Attr()
        attr['namespace'] = obj.namespace
        attr['attr_name'] = self.attr_name
        return attr

    def __set__(self, obj, value):
        obj.root.context.nodes.append(u'{0}.{1} = {2}'.format(obj.namespace, self.attr_name, escape_value(value)))

    def __del__(self, obj):
        pass


class Node(object):
    
    html = AttrAccessor('innerHTML')
    text = AttrAccessor('innerText')

    def __init__(self, selector=None, parent=None, level=0, multi=False, var=None, namespace=''):

        self.selector = selector
        self.parent = parent
        self.level = level+1
        self.multi = multi
        self.nodes = []
        self.var = var
        self.namespace = namespace and namespace+namespace[-1] or var[0]

        self.delim = u'\n' + '\t'*self.level

        if not parent:
            self.root = self
            self.context = self
        else:
            self.root = parent.root

    def __enter__(self):
        self.root.context = self

        return self

    def __exit__(self, e,v,t):
        self.root.context = self.parent
        if self.parent:
            self.parent.add_node(self)

    def add_node(self, node):
        self.nodes.append(node.render())

    def get_selector(self):
        if self.selector:
            return u'{0}.querySelector{1}("{2}")'.format(self.parent.namespace, self.multi and 'All' or '', self.selector)

        if self.parent:
            return u'{0}.{1}'.format(self.parent.namespace, self.var)
        else:
            return self.var

    def e(self, selector):
        return Node(selector=selector, parent=self, level=self.level, multi=False, namespace=self.namespace)

    def el(self, selector):
        return Node(selector=selector, parent=self, level=self.level, multi=True, namespace=self.namespace)

    def render(self):
        return u'(function({0}){{{3}{2};{4}}})({1})'.format(
                self.namespace, 
                self.get_selector(),                 
                ';{0}'.format(self.delim).join(self.nodes),
                self.delim,
                self.delim[:-1]
            )


class Window(Node):

    @property
    def document(self):
        return Node(var='document', parent=self, level=1)


class PJSDescriptor(object):

    def __init__(self, klass, kwargs):
        self.klass = klass
        self.kwargs = kwargs

    def __get__(self, obj, owner):
        node = self.klass(**self.kwargs)
        obj._node = node
        return node

    def __set__(self, obj, value):
        pass

class PJS(object):
        
    window = PJSDescriptor(Window, {'var':'window'})
    document = PJSDescriptor(Node, {'var':'document'})
    _node = None

    def __init__(self):
        self._node = None
        
    def __enter__(self):
        return self

    def __exit__(self, e, v, t):
        pass

    def __getattr__(self, attr):
        if vars(self).has_key(attr):
            return vars(self)[attr]
        else: 
            return Attr(attr_name=attr, namespace=None)

    def __setattr__(self, attr, value):        
        if attr in ['_node', 'window', 'document']:
            vars(self)[attr] = value
        else:
            self._node.context.nodes.append(u'{0} = {1}'.format(attr, escape_value(value)))

    def var(self, var):
        defines = []
        if type(var) == list:          
            defines.extend(var)  
        elif type(var) == dict:
            defines.extend(['{0} = {1}'.format(name, escape_value(value)) for name, value in var.items()])
        else:
            defines.append(var)

        self._node.context.nodes.append(u'var {0}'.format(', '.join(defines)))

    def render(self):
        return self._node.render()



#coding=utf-8
import re

"""
ar must be an object of ActiveRecord or derived from it

ForeignKey(ar=Cafe)
TagStrip()
XssStrip()
Email()
Integer(op='<', arg=1500)
Integer()
Numeric()
Double(op='>', arg=15.0)
Double()
Required()
Length(op='>=', arg=100)
Trim()
Password()
Coords()
Unique(ar=Place, field='title')
Cast(op=cast_operator)
"""

class ValidationError(Exception):
    pass

class Security(object):
    name = ''
    value = None
    this = None
    errors = []
    
    pod = lambda x : not isinstance(x, type([])) and not isinstance(x, type({}))
    
    @property
    def val(self):
        return self.value
    
    def __init__(self, **kwargs):
        self.this = kwargs.get('this')
        
    def setvalue(self, v):
        self.value = v
    
    def setobject(self, o):
        self.this = o
    
    def perform(self):
        pass
    
    def __call__(self):
        self.errors = []
        return self.perform()
    
    def relop(self, reloper, source, target):
        
        if reloper == '<':
            return source < target
        elif reloper == '>':
            return source > target
        elif reloper == '=':
            return source == target
        elif reloper == '>=':
            return source >= target
        elif reloper == '<=':
            return source <= target
        
        return False
    
    def regex_match(self, regex):
        bad = 0
        if isinstance(self.value, type([])):
            for v in self.value:
                if not regex.match(v):
                    bad += 1
        elif isinstance(self.value, type({})):
            for v in self.value.itervalues():
                if not regex.match(v):
                    bad += 1
        else:
            if not regex.match(self.value):
                bad += 1
        
        return bad == 0
    
    errors_messages = {
        'fk':u'Элемент не найден',
        'double' :u'Параметр не является числом',
        'double_param':u'Значение должно быть %(relop)s %(arg)g',
        'email':u'Адрес электронной почты не корректен',
        'integer':u'Значение не является целым числом',
        'integer_param':u'Значение должно быть %(relop)s %(arg)d',
        'numeric':u'Значение не является числом',
        'required':u'Поле обязательно для заполнения',
        'password_length':u'Длина пароля короткая',
        'password_mismatch':u'Пароли не совпали',
        'length': u'Размер должен быть %(relop)s %(arg)d',
        'unique': u'Значение не может быть использовано',
        'coords_bad_range':u'Координаты заданы неверно',
        'photo': u'Изображение не загружено',
        'regex': u'Неверный формат'
    }
    def format_error(self, token, relop=None, arg=None):
        self.errors.append(self.errors_messages[token] % {'relop':relop,'arg':arg})

class ForeignKey(Security):
    name = 'fk'
    def __init__(self, **kwargs):
        super(ForeignKey, self).__init__(**kwargs)
        self.ar = kwargs.get('ar', None)
        
    def setobject(self, o):
        if not self.ar:
            self.ar = o.__class__
        super(ForeignKey, self).setobject(o)
    
    def is_number(self, num):
        return re.compile(r'^\d+$').match(num) != None
    
    def perform(self):
        
        if isinstance(self.value, type([])):
            new_value = []
            for v in self.value:                
                if not self.is_number(v) or int(v) != 0 and not self.ar.is_exists(int(v)):
                    new_value.append('0')
                else:
                    new_value.append(v)
            self.value = new_value
        elif isinstance(self.value, type({})):
            new_value = {}
            for k, v in self.value.iteritems():
                if not self.is_number(v) or int(v) != 0 and not self.ar.is_exists(int(v)):
                    new_value[k] = '0'
                else:
                    new_value[k] = v
                self.value = new_value
        else:
            if not self.is_number(self.value) or int(self.value) != 0 and not self.ar.is_exists(int(self.value)):
                self.value = '0'

        return True
        
        
class TagStrip(Security):
    name = 'tagstrip'
    def perform(self):
        regex_expr = '(\>|\<\/?)+'
        if isinstance(self.value, type([])):
            new_value = []
            for v in self.value:
                new_value.append(re.sub(regex_expr, '', v, re.I | re.M))
            self.value = new_value
        elif isinstance(self.value, type({})):
            new_value = {}
            for k, v in self.value.iteritems():
                new_value[k] = re.sub(regex_expr, '', v, re.I | re.M)
            self.value = new_value
        else:
            self.value = re.sub(regex_expr, '', self.value, re.I | re.M)
        
        return True
        
        
class XssStrip(Security):
    name = 'xssstrip'
    def perform(self):
        regex_expr = r'<(script|object|embed|iframe|applet|body|frameset|frame|head|html|form|vbscript)[^>]*?>(.*?)</\1>'
        if isinstance(self.value, type([])):
            new_value = []
            for v in self.value:
                new_value.append(re.sub(regex_expr, '', v, re.I | re.M | re.S))
            self.value = new_value
        elif isinstance(self.value, type({})):
            new_value = {}
            for k, v in self.value.iteritems():
                new_value[k] = re.sub(regex_expr, '', v, re.I | re.M | re.S)
            self.value = new_value
        else:
            self.value = re.sub(regex_expr, '', self.value, re.I | re.M | re.S)
        
        return True
    
    
class Email(Security):
    name = 'email'
    def perform(self):
        regex = re.compile('^([a-z0-9\+_\-]+)(\.[a-z0-9\+_\-]+)*@([a-z0-9\-]+\.)+[a-z]{2,6}$', re.I | re.U)
        result = self.regex_match(regex)
        
        if not result:
            self.format_error('email')
            
        return result
    
    
class Integer(Security):
    name = 'integer'
    def __init__(self, **kwargs):
        super(Integer, self).__init__(**kwargs)
        self.op = kwargs.get('op', None)
        self.arg = kwargs.get('arg', None)
        
    def perform(self):
        result = self.regex_match(re.compile(r'^[0-9]+$'))
        
        if not self.op and not self.arg and not result:
            self.format_error('integer')
        elif self.op and self.arg and not result:
            self.format_error('integer')
        if self.op and self.arg != None and result:
            bad = 0
            if isinstance(self.value, type([])):
                for v in self.value:
                    if not self.relop(self.op, int(v), self.arg):
                        bad += 1
                        self.format_error('integer_param', self.op, self.arg)
            elif isinstance(self.value, type({})):
                for v in self.value.values():
                    if not self.relop(self.op, int(v), self.arg):
                        bad += 1
                        self.format_error('integer_param', self.op, self.arg)
            else:
                if not self.relop(self.op, int(self.value), self.arg):
                    bad += 1
                    self.format_error('integer_param', self.op, self.arg)
            
            result = bad == 0
        
        return result
    
    
class Numeric(Security):
    name = 'numeric'
    def perform(self):
        result = self.regex_match(re.compile(r'^\d+|\d+\.\d+|\.\d+$'))
        
        if not result:
            self.format_error('numeric')
        
        return result
    
    
class Double(Security):
    name = 'float'
    def __init__(self, **kwargs):
        super(Double, self).__init__(**kwargs)
        self.op = kwargs.get('op', None)
        self.arg = kwargs.get('arg', None)
        
    def perform(self):
        result = self.regex_match(re.compile(r'^-?\d+|\d+\.\d+|\.\d+$'))
        
        if not self.op and not self.arg and not result:
            self.format_error('double')
        
        if self.op and self.arg and result:
            bad = 0
            if isinstance(self.value, type([])):
                for v in self.value:
                    if not self.relop(self.op, float(v), self.arg):
                        bad += 1
                        self.format_error('double_param', self.op, self.arg)
            elif isinstance(self.value, type({})):
                for v in self.value.values():
                    if not self.relop(self.op, float(v), self.arg):
                        bad += 1
                        self.format_error('double_param', self.op, self.arg)
            else:
                if not self.relop(self.op, float(self.value), self.arg):
                    bad += 1
                    self.format_error('double_param', self.op, self.arg)
            
            result = bad == 0
        
        return result
    
    
class Required(Security):
    name = 'required'
    def perform(self):
        
        bad = 0
        
        if isinstance(self.value, type([])):
            for v in self.value:
                bad += 1 if len(v) == 0 else 0
        elif isinstance(self.value, type({})):
            for v in self.value.values():
                bad += 1 if len(v) == 0 else 0
        else:
            bad += 1 if len(self.value) == 0 else 0
        
        if bad:
            self.format_error('required')
            
        return bad == 0
    
    
class Length(Security):
    name = 'length'
    def __init__(self, **kwargs):
        super(Length, self).__init__(**kwargs)
        self.op = kwargs.get('op')
        self.arg = kwargs.get('arg')
        
    def perform(self):
        bad = 0
        if isinstance(self.value, type([])):
            for v in self.value:
                if not self.relop(self.op, len(v), self.arg):
                    bad += 1
                    self.format_error('length', self.op, self.arg)
        elif isinstance(self.value, type({})):
            for v in self.value.itervalues():
                if not self.relop(self.op, len(v), self.arg):
                    bad += 1
                    self.format_error('length', self.op, self.arg)
        else:
            if not self.relop(self.op, len(self.value), self.arg):
                bad += 1
                self.format_error('length', self.op, self.arg)
        
        return bad == 0
    
    
class Trim(Security):
    name = 'trim'
    def perform(self):
        if isinstance(self.value, type([])):
            new_value = []
            for v in self.value:
                new_value.append(v.strip())
            self.value = new_value
        elif isinstance(self.value, type({})):
            new_value = {}
            for k, v in self.value.iteritems():
                new_value[k] = v.strip()
            self.value = new_value
        else:
            self.value = self.value.strip()
        
        return True
    
    
class Password(Security):
    name = 'password'
    def perform(self):
        if (isinstance(self.value, type([])) or isinstance(self.value, type({}))) and len(self.value) == 2:
            
            if isinstance(self.value, type([])):
                first, second = self.value[:2]
            else:
                first, second = list(self.value.values())[:2]
            if len(first) < 6:
                self.format_error('password_length')
                return False
            
            if first != second:
                self.format_error('password_mismatch')
                return False
            
            self.value = first
            
            return True
        
        self.format_error('password_mismatch')
        return False
    
    
class Coords(Security):
    name = 'coords'
    def perform(self):
        if isinstance(self.value, dict):
            self.value = self.value.values()
        
        if isinstance(self.value, list) and len(self.value) == 2:
            if not self.value[0] and not self.value[1]:
                return True
            
            val0 = float(self.value[0])
            val1 = float(self.value[1])
            
            if val0 <= 180.0 and val0 >= -180.0 and val1 <= 180.0 and val1 >= -180.0:
                self.value[0] = val0
                self.value[1] = val1
                return True
            
            self.format_error('coords_bad_range')
                
        return False
    
    
class Unique(Security):
    name = 'unique'
    def __init__(self, **kwargs):
        super(Unique, self).__init__(**kwargs)
        self.ar = kwargs.get('ar', None)
        self.field = kwargs.get('field')
        
    def setobject(self, o):
        if not self.ar:
            self.ar = o.__class__
        super(Unique, self).setobject(o)
        
    def perform(self):
        found = False
        cond = {}
        
        if self.this.has_key('_id'):
            cond = {'_id': {'$ne': self.this.get('_id')}}
        
        if isinstance(self.value, type([])):
            for v in self.value:
                cond[self.field] = v
                if self.ar.find_first(cond):
                    found = True
        elif isinstance(self.value, type({})):
            for v in self.value.itervalues():
                cond[self.field] = v
                if self.ar.find_first(cond):
                    found = True
        else:
            cond[self.field] = self.value
            if self.ar.find_first(cond):
                found = True
        
        if found:
            self.format_error('unique')
        
        return not found
    
    
class Cast(Security):
    def __init__(self, **kwargs):
        super(Cast, self).__init__(**kwargs)
        self.cast_oper = kwargs.get('op', None)
        
    def perform(self):
        
        self.value = self.performer(self.value)
        
        return True
        
    def performer(self, value):
        
        pod = lambda x : not isinstance(x, type([])) and not isinstance(x, type({}))
        
        if isinstance(value, type([])):
            new_value = []
            for v in value:
                new_value.append(self.cast_oper(v) if pod(v) else self.performer(v))
            return new_value
        elif isinstance(value, type({})):
            new_value = {}
            for k, v in value.iteritems():
                new_value[k] = self.cast_oper(v) if pod(v) else self.performer(v)
            return new_value
        else:
            return self.cast_oper(value.encode('ascii')) if value else 0

class Photo(Security):
    def __init__(self, **kwargs):
        super(Photo, self).__init__(**kwargs)
        self.required = kwargs.get('req', False)
        
    def perform(self):
        result = True
        
        for x in self.value.values():
            if x.has_key('hash') and x.has_key('ext'):
                if self.required and not x['hash'] and not x['ext']:
                    result = False
            else:
                result = False
        
        if not result:
            self.format_error('photo')
            
        return result

class Regex(Security):
    def __init__(self, arg):
        super(Regex, self).__init__()
        self.regex = arg
        
    def perform(self):
        ret = self.regex_match(re.compile(self.regex, re.UNICODE))
        if not ret:
            self.format_error('regex')
        return ret



#coding=utf-8

import smtplib
from datetime import datetime
from hashlib import md5
import sys, re
from .misc import *
from .parts import *

from collections import OrderedDict as odict

class Mimemail():
    
    def __init__(self, **kwargs):
        self.headers = odict()
        self.headers['MIME-Version'] = '1.0'
        self.headers['From'] = MM_DEFAULT_FROM
        self.headers['Date'] = datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT')
        
        self.body = ''
        self.html = None
        self.text = None
        self.images = []
        self.attachments = []
        self.charset = 'UTF-8'
        self.recipients = {}
        self.from_email = 'root@localhost'
        self.kw = kwargs
    
    def set_from(self, from_email, from_name):
        self.headers['From'] = '%s <%s>' % (encode_header(from_name, self.charset), from_email)
        self.from_email = from_email
        
    def set_html(self, html):
        self.html = html
        
    def set_text(self, text):
        self.text = text
    
    def add_image(self, image):
        self.images.append(image)
        
    def add_attachment(self, att):
        self.attachments.append(att)
        
    def set_subject(self, subject):
        self.subject = subject

    def create_images_part(self, boundary):
        lines = []
        for image in self.images:
            lines.extend([
                MM_DEFAULT_CRLF,
                '--%s%s' % (boundary, MM_DEFAULT_CRLF),
                image.get_part_header(),
                MM_DEFAULT_CRLF,
                MM_DEFAULT_CRLF,
                image.get_part_body()
            ])
            
        return ''.join(lines)
    
    def create_attachments_part(self, boundary):
        lines = []
        for att in self.attachments:
            lines.extend([
                MM_DEFAULT_CRLF,
                '--%s%s' % (boundary, MM_DEFAULT_CRLF),
                att.get_part_header(),
                MM_DEFAULT_CRLF,
                MM_DEFAULT_CRLF,
                att.get_part_body()
            ])
        
        return ''.join(lines)
        
    def build(self):
        has_html = self.html != None
        has_text = self.text != None
        has_img  = len(self.images) > 0
        has_att  = len(self.attachments) > 0
        
        if has_text and not has_html:
            self.html = MimemailPartHtml(re.sub(r'\n', '<br>', self.text.plain_content, re.M | re.S), charset = self.charset)
        elif has_html and not has_text:
            self.text = MimemailPartText(re.sub(r'<|>|/', '', self.html.plain_content, re.M | re.S | re.U), charset = self.charset)
        elif not has_html and not has_text and not has_att:
            raise MimemailException('An email has no content to send')
            
        if has_img:
            for image in self.images:
                src = image.get_file_path()
                dst = 'cid:' + image.get_image_cid()
                self.html.plain_content = self.html.plain_content.replace(os.path.basename(src), dst)
        
        boundary = 'alt_' + gen_boundary_hash()
        self.headers['Content-Type'] = 'multipart/alternative; boundary="' + boundary + '"'
        
        self.body = ''.join([
            '--%s%s' % ( boundary, MM_DEFAULT_CRLF ),
            self.text.get_part_header(),
            MM_DEFAULT_CRLF,
            MM_DEFAULT_CRLF,
            self.text.get_part_body(),
            '%s--%s%s' % ( MM_DEFAULT_CRLF, boundary, MM_DEFAULT_CRLF ),
            self.html.get_part_header(),
            MM_DEFAULT_CRLF,
            MM_DEFAULT_CRLF,
            self.html.get_part_body(),
            '%s--%s--%s%s' % ( MM_DEFAULT_CRLF, boundary, MM_DEFAULT_CRLF, MM_DEFAULT_CRLF )
        ])
        
        if has_img:
            boundary = 'rel_' + gen_boundary_hash()
            self.body = ''.join([
                '--%s%s' % ( boundary, MM_DEFAULT_CRLF ),
                'Content-Type: %s%s%s' % (self.headers['Content-Type'], MM_DEFAULT_CRLF, MM_DEFAULT_CRLF),
                self.body,
                self.create_images_part(boundary),
                '%s--%s--%s%s' % ( MM_DEFAULT_CRLF, boundary, MM_DEFAULT_CRLF, MM_DEFAULT_CRLF )
            ])
            self.headers['Content-Type'] = 'multipart/related; boundary="%s"' % (boundary)
            
        if has_att:
            boundary = 'att_' + gen_boundary_hash()
            self.body = ''.join([
                '--%s%s' % (boundary, MM_DEFAULT_CRLF ),
                'Content-Type: %s%s%s' % (self.headers['Content-Type'], MM_DEFAULT_CRLF, MM_DEFAULT_CRLF),
                self.body,
                self.create_attachments_part(boundary),
                '%s--%s--%s%s' % ( MM_DEFAULT_CRLF, boundary, MM_DEFAULT_CRLF, MM_DEFAULT_CRLF )
            ])
            self.headers['Content-Type'] = 'multipart/mixed; boundary="%s"' % (boundary)
            
        self.headers['Message-ID'] = self.gen_message_id()
        
        if hasattr(self, 'subject'):
            self.headers['Subject'] = encode_header(self.subject, self.charset)
        
    def gen_message_id(self):
        return '<%s.%08x@%s>' % (datetime.datetime.now().strftime('%Y%m%d%H%M%S'), random.randint(0, sys.maxint), self.kw.get('host', 'localhost'))

    def add_recipient(self, email, name = None):
        self.recipients[email] = name if name else email

    def send(self):
        self.build()
        
        extra_headers = self.get_extra_headers()
        
        for email, name in self.recipients.iteritems():
            message = '%s%sTo: %s <%s>%s%s%s' % (extra_headers, MM_DEFAULT_CRLF, encode_header(name, self.charset), email, MM_DEFAULT_CRLF, MM_DEFAULT_CRLF, self.body)
            s = smtplib.SMTP(self.kw.get('smtp_relay', '127.0.0.1'))
            s.sendmail(self.from_email, email, message)
            s.quit()
        
    def get_extra_headers(self):
        return MM_DEFAULT_CRLF.join([ '%s: %s' % (k, v) for k,v in self.headers.iteritems() ])





from tornado import web, gen
from tornado.escape import parse_qs_bytes

from .views import JScriptView, HtmlView, JsonView

class Behavior:
    def __init__(self, handler, *args, **kwargs):
        self.handler = handler
        self.args = args
        self.kwargs = kwargs

    def respond(self):
        pass


class Redirect(Behavior):
    def respond(self):
        self.handler.redirect(self.kwargs.get('url'))


class Error(Behavior):
    def respond(self):
        self.handler.send_error(self.kwargs.get('code'))


class Handler(web.RequestHandler):
    http_get_view_adapter = None
    http_post_view_adapter = None

    def context(self):
        return {}

    def initialize(self, *args, **kwargs):
        self.install_translator()

    def install_translator(self):
        self.translator = self.application.translators.get(self.get_user_locale(), object())

    def _(self, message):
        return getattr(self.translator, 'ugettext', lambda x: x)(message)

    def _parse_arguments(self, src):
        arguments = {}
        for key, value in parse_qs_bytes(src, keep_blank_values=True).items():

            value = [x.decode('utf-8') for x in value]

            v = value
            if key.endswith('[]'):
                key = key[:-2]
            else:
                v = value[0]
            td = arguments
            parts = key.split('[')
            for i, tkey in enumerate(parts):
                tkey = tkey.strip('[]')
                if i+1 < len(parts):
                    td = td.setdefault(tkey, {})
            try:
                td[key.split('[')[-1].strip('[]')] = v
            except:
                pass

        return arguments

    @property
    def arguments(self):
        if not hasattr(self, '_parsed_arguments'):
            self._parsed_arguments = self._parse_arguments(self.request.query)

            if self.request.method == 'POST':
                self._parsed_arguments.update(self._parse_arguments(self.request.body))

        return self._parsed_arguments

    def send_response(self, v):
        for hname, hvalue in v.headers.iteritems():
            self.set_header(hname, hvalue)
        self.write(v.render())

    def complete(self, data):
        if isinstance(data, Behavior):
            data.respond()
        else:
            view = self.http_view_adapter(self, self.context, data) if self.http_view_adapter else self.view_factory(self.context, data)
            self.send_response(view)

        self.finish()

    @web.asynchronous
    @gen.coroutine
    @web.addslash
    def get(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

        self.context = yield self.get_context()
        can = yield self.can_get()

        if isinstance(can, Behavior):
            can.respond()
        elif not can:
            self.send_error(403)
        else:
            self.http_view_adapter = self.http_get_view_adapter
            __dummy = yield self.do_get()

    @web.asynchronous
    @gen.coroutine
    def post(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

        self.context = yield self.get_context()
        can = yield self.can_post()

        if isinstance(can, Behavior):
            can.respond()
        elif not can:
            self.send_error(403)
        else:
            self.http_view_adapter = self.http_post_view_adapter
            __dummy = yield self.do_post(context)


    def do_get(self):
        return {}

    def can_get(self):
        return True

    def do_post(self):
        return {}

    def can_post(self):
        return True

    def __init__(self, *args, **kwargs):
        super(Handler, self).__init__(*args, **kwargs)

        self.get_context  = self.application.asynchronize(self.get_context)

        self.can_get  = self.application.asynchronize(self.can_get)
        self.can_post = self.application.asynchronize(self.can_post)

        self.do_get   = gen.coroutine(self.do_get)
        self.do_post  = gen.coroutine(self.do_post)

        self.initkw = kwargs

    def view_factory(self, context, data):
        accept = self.request.headers.get('Accept', 'text/html')

        if 'application/javascript' in accept:
            return JScriptView(self, context, data)
        if 'application/json' in accept:
            return JsonView(self, context, data)
        else:
            return HtmlView(self, context, data)





import json
from jinja2 import Environment, FileSystemLoader

class View:
    headers = {}
    body = ''
    
    def __init__(self, *args, **kwargs):
        
        self.handler = args[0]
        self.context = args[1]
        self.data    = args[2]
        
        self._ = self.handler._
    
    def no_cache_header(self):
        self.headers['Pragma']  = 'no-cache'
        self.headers['Expires'] = '0'
        self.headers['Cache-Control'] = 'must-revalidate, post-check=0, pre-check=0'
        
    def render(self):
        return self.body
    
class JsonView(View):

    def __init__(self, *args, **kwargs):
        self.headers['Content-Type'] = 'application/json'
        View.__init__(self, *args, **kwargs)
    
    def render(self):
        return json.dumps(self.data)
    


class Path(list):
    @property
    def next(self):
        return self.pop(0)


class HtmlView(View):

    def __init__(self, *args, **kwargs):
        View.__init__(self, *args, **kwargs)
        
        self.headers['Content-Type'] = 'text/html; charset="utf-8"'
        self.no_cache_header()
        
        ext = ['jinja2.ext.with_', 'jinja2.ext.i18n','jinja2.ext.do']
        if not self.handler.settings.get('debug'):
            ext.append('ramen.jinja2htmlcompress.HTMLCompress')
        
        self.jinja2 = Environment(
            loader = FileSystemLoader(self.handler.settings['template_path']),
            extensions = ext,
            trim_blocks = True
        )
    
    def render(self):
        data = self.data
        data.update(self.context)
        data['_'] = self._
        
        path = ''
        path_list = Path([])
        for namespace in self.handler.initkw.pop('namespaces', []):
            path = '%s%s/' % (path, namespace)
            path_list.append('%s_wrapper.html' % path)
        
        path_list.append(self.handler.template_name)
        data['template'] = path_list
        
        t = self.jinja2.get_template(self.handler.template_root)
        return t.render(**data)

class JScriptView(View):

    def __init__(self, *args, **kwargs):
        self.headers['Content-Type'] = 'application/x-javascript; charset="UTF-8"'
        View.__init__(self, *args, **kwargs)



from .fields import *
from .proxy import DBProxy

from tornado import gen

__all__ = [
    'Record'
]


class MetaRecord(type):
    """
    MetaRecord
    """

    def __new__(mcs, name, bases, attrs):

        klass = type.__new__(mcs, name, bases, attrs)

        fields = []
        for base in klass.__mro__:
            for attr_name, attr in base.__dict__.items():
                if isinstance(attr, Field):
                    attr.set_key(attr_name)

                    if isinstance(attr, ForeignKey):
                        attr.install_relation(klass, name)

                    setattr(klass, attr_name, attr)
                    fields.insert(0, attr)

        fields = [f for i, f in enumerate(fields) if not f in fields[:i]]

        setattr(klass, '_fields', fields)

        return klass

def gen_task(fn):
    def wrapper(*args, **kwargs):
        return gen.Task(fn, *args, **kwargs)
    return wrapper

class Query(object):

    def __init__(self, klass, db):
        self.klass = klass
        self.db = db

        self._i = 0
        self._total = 0

        self._col = self.db[self.klass._collection]

        #self.statement = connection.get_statement()
        #self.statement.set_FROM(self.klass._collection)

        self.objects = []

        self._loaded = False
        self._query = {}
        self._limit = 100
        self._skip = 0
        self._sort = []


    def fetch_wrapper(self, obj, error, callback):
        self.objects = obj
        callback(self)

    @gen_task
    def fetch(self, callback):
        if not self._loaded:
            self._col.find(
                self._query,
                skip=self._skip,
                limit=self._limit,
                sort=self._sort
            ).to_list(callback = lambda o,e:self.fetch_wrapper(o, e, callback))
            self._loaded = True

    def __iter__(self):
        self._i = 0
        self._total = len(self.objects)
        return self

    def next(self):
        if self._i < self._total:
            self._i += 1
            return self.klass.init(self.objects[self._i - 1])
        else:
            raise StopIteration

    def result_wrapper(self, obj, error, callback):
        callback(self.klass.init(obj))

    @gen_task
    def get(self, *args, **kwargs):

        callback = kwargs.pop('callback')

        if len(args) == 1:
            obj = self._col.find_one(args[0], callback=lambda o,e:self.result_wrapper(o, e, callback))
        else:
            obj = self._col.find_one(kwargs, callback=lambda o,e:self.result_wrapper(o, e, callback))

    def filter(self, _query=None, **kwargs):
        query = kwargs or _query
        self._query.update(query)
        return self

    def skip(self, skip):
        self._skip = skip
        return self

    def limit(self, limit):
        self._limit = limit
        return self

    def sort(self, **kwargs):
        self._sort.extend(kwargs.items())

        return self



class ConnectionDescriptor(object):

    client = None

    def __init__(self):
        self.client = DBProxy().db

    def __get__(self, obj, owner):
        return self.client


class classproperty(object):
    def __init__(self, getter):
        self.getter = getter
    def __get__(self, instance, owner):
        return self.getter(owner)

class Record(dict):
    """
    Record
    """

    __metaclass__ = MetaRecord
    _collection = None
    _dbclient = ConnectionDescriptor()

    id = Field(default=None, alias='_id')

    def __init__(self, **kwargs):
        super(Record, self).__init__()
        self.update(kwargs)

        for f in self._fields:
            f.init(self)

    @classmethod
    def init(cls, attrs):
        obj = cls()
        obj.update(attrs)
        return obj

    @classproperty
    def Q(cls):
        return Query(cls, cls._dbclient)

    def _save_callback(self, new_id, error, callback):
        callback(new_id)

    def _generate_id_callback(self, next, error, callback):
        self.id = next['next']
        self._dbclient[self._collection].insert(self, callback=lambda o,e:self._save_callback(self.id,e,callback))

    @gen_task
    def save(self, callback):

        is_valid = all(f.validate(self) for f in self._fields)

        if is_valid:
            if self.id == None:
                self._dbclient['_sequences'].find_and_modify(
                    {'_id':self._collection},
                    {'$inc':{'next':1}},
                    True,
                    new=True,
                    callback=lambda o,e:self._generate_id_callback(o,e,callback)
                )
            else:
                self._dbclient[self._collection].update(
                    {'_id' : self.id},
                    self,
                    upsert=True,
                    callback=lambda o,e:self._save_callback(self.id,e,callback)
                )

        callback(is_valid)





#!/usr/bin/env python

import os

try:
    from setuptools.core import setup
    from setuptools.command.install import INSTALL_SCHEMES
except ImportError:
    from distutils.core import setup
    from distutils.command.install import INSTALL_SCHEMES

dependencies = ['motor', 'tornado']

def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

# Tell distutils not to put the data_files in platform-specific installation
# locations. See here for an explanation:
# http://groups.google.com/group/comp.lang.python/browse_thread/thread/35ec7b2fed36eaec/2105ee4d9e8042cb
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']


packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
django_dir = 'ramen'

for dirpath, dirnames, filenames in os.walk(django_dir):
    # Ignore PEP 3147 cache dirs and those whose names start with '.'
    dirnames[:] = [d for d in dirnames if not d.startswith('.') and d != '__pycache__']
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])


setup(
    name='ramen',
    version='0.0.2',
    description="Tornado based simple web framework",
    long_description=open('README.md').read(),
    author='Anton Gorskiy',
    author_email='gorskiy.anton@gmail.com',
    url='http://gorskiy.kz/ramen',
    packages=packages,
    data_files=data_files,
    license='Apache v2',
    install_requires=dependencies,
    scripts = ['ramen/bin/ramen'],
    classifiers = [
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP :: HTTP Servers'
    ]
)




from tornado import httpserver, ioloop, web
from ramen.urls import include
from ramen.decorators import asynchronize
import os
import gettext

class Bootstrap(object):

    def __init__(self, port, **settings):
        
        settings['template_path'] = os.path.join(os.getcwd(), 'templates')
        
        app = web.Application(include(r'^/', 'urls'), **settings)
        server = httpserver.HTTPServer(app)
        if settings['debug']:
            server.listen(port)
        else:
            server.bind(port)
            server.start(0)
        
        ioinst = ioloop.IOLoop.instance()
        
        app.asynchronize = asynchronize(ioinst)
        app.translators = self.install_translators()
        
        try:
            print 'Starting on 0.0.0.0:%s' % port
            ioinst.start()
        except:
            ioinst.stop()
    
    def install_translators(self):
        locale_path = os.path.join(os.getcwd(), 'locales')
        translators = {}
        
        if os.path.exists(locale_path) and os.path.isdir(locale_path):
            for lang in os.listdir(locale_path):
                lc_messages = os.path.join(locale_path, lang, 'LC_MESSAGES')
                if os.path.isdir(lc_messages) and os.path.isfile(os.path.join(lc_messages, 'messages.mo')):
                    try:
                        trans = gettext.translation('messages', os.path.join(os.getcwd(), 'locales'), [lang])
                        translators[lang] = trans
                    except:
                        print 'Translator fail on load %s' % os.path.join(lc_messages, 'messages.mo')
            
        return translators


#coding=utf-8

from .misc import *
import os
from collections import OrderedDict as odict

class MimemailPart(object):
    image_types = {
        'gif':  'image/gif',
        'jpg':  'image/jpeg',
        'jpeg': 'image/jpeg',
        'jpe':  'image/jpeg',
        'png':  'image/png',
        'tif':  'image/tiff',
        'tiff': 'image/tiff',
        'swf':  'application/x-shockwave-flash'
    }
    
    def __init__(self):
        self.part_type = ''
        self.boundary = ''
        self.body = ''
        self.headers = odict
        self.plain_content = ''
        self.charset = 'UTF-8'
        
    def get_part_header(self):
        if len(self.headers):
            return MM_DEFAULT_CRLF.join([ '%s: %s' % (k,v) for k,v in self.headers.iteritems() ])
        return ''
        
    def get_part_body(self):
        return self.body
    
    def get_part_type(self):
        return self.part_type
    
    
class MimemailPartImage(MimemailPart):
    def __init__(self, filepath, content_type = None, content = None):
        super(MimemailPartImage, self).__init__()
        self.part_type = 'IMAGE'
        self.cid = gen_boundary_hash()
        self.image_file = filepath
        name = os.path.basename(filepath)
        if content_type:
            self.headers['Content-Type'] = content_type
        else:
            ext = name[name.rfind('.')+1:].lower()
            if ext in self.image_types:
                self.headers['Content-Type'] = '%s; name="%s"' % (self.image_types[ext], name)
            else:
                self.headers['Content-Type'] = 'application/octet-stream; name="%s"' % name
                
        self.headers['Content-Transfer-Encoding'] = 'base64'
        self.headers['Content-Disposition'] = 'inline; filename="%s"' % name
        self.headers['Content-ID'] = '<%s>' % self.cid
        
        if not content:
            f = open(filepath, 'rb')
            self.body = f.read()
            f.close()
        else:
            self.body = content
        
        self.body = encode_content(self.body)
        
    def get_image_cid(self):
        return self.cid
    
    def get_file_path(self):
        return self.image_file
    
    
class MimemailPartAttachment(MimemailPart):
    def __init__(self, filepath, content_type = 'application/octet-stream', content = None):
        super(MimemailPartAttachment, self).__init__()
        self.part_type = 'ATTACHMENT'
        
        if not content:
            f = open(filepath, 'rb')
            self.body = f.read()
            f.close()
        else:
            self.body = content
        
        self.body = encode_content(self.body)
        
        self.headers['Content-Type'] = '%s; name="%s"' % (content_type, os.path.basename(filepath))
        self.headers['Content-Transfer-Encoding'] = 'base64'
        self.headers['Content-Disposition'] = 'attachment; filename="%s"' % os.path.basename(filepath)
        
    
class MimemailPartText(MimemailPart):
    def __init__(self, text, **kwargs):
        super(MimemailPartText, self).__init__()
        self.part_type = 'TEXT'
        self.headers['Content-Type'] = 'text/plain; charset="%s"' % kwargs.get('charset', 'UTF-8')
        self.headers['Content-Transfer-Encoding'] = 'base64'
        self.plain_content = text
        self.body = encode_content(text)
    
    
class MimemailPartHtml(MimemailPart):
    def __init__(self, html, **kwargs):
        super(MimemailPartHtml, self).__init__()
        self.part_type = 'HTML'
        self.headers['Content-Type'] = 'text/html; charset="%s"' % kwargs.get('charset', 'UTF-8')
        self.headers['Content-Transfer-Encoding'] = 'base64'
        self.plain_content = html

    def get_part_body(self):
        return encode_content(self.plain_content)



__all__ = [
    'Q'
]

def operand_maker(operand):
    def func(self, value):
        return self.value({operand: value})
    return func

def list_maker(operand):
    def func(self, q):
        if not isinstance(q, Query):
            raise TypeError('Expected Query object, got: {}'.format(q))
        return Query({operand: [self, q]})
    return func

class QueryError(Exception): pass

class Query(dict):

    def __init__(self, *a, **k):
        super(Query, self).__init__(*a, **k)
        self.field = None

    def _dup(self):
        q = Query(self)
        q.field = self.field
        return q

    def value(self, value):
        if self.field is None:
            raise QueryError('Cannot add a value to a query bound to multiple fields')
        q = self._dup()
        if isinstance(value, dict) and self.field in self:
            if not isinstance(q[self.field], dict):
                raise QueryError('Can only add keys to documents, not to: {}'.format(q[self.field]))
            self._verify_not_overrides(q[self.field], value)
            q[self.field] = Query(q[self.field], **value)
        else:
            q[self.field] = value
        return q

    def __getattr__(self, field):
        if field == 'id':
            field = '_id'
        if self:
            raise QueryError('Generating a Query on a sub document will abandon current query')
        q = self._dup()
        if self.field is not None:
            q.field += '.' + field
        else:
            q.field = field
        return q

    def _verify_not_overrides(self, d1, d2):
        overriden = set(d1) & set(d2)
        if overriden:
            raise QueryError("The following fields will be overriden: {} for Queries: {}, {}".format(list(overriden), d1, d2))

    def __add__(self, other):
        self._verify_not_overrides(self, other)
        return Query(self, **other)

    def __invert__(self):
        return Query({'$not': self})

    def mod(self, d, m):
        return self.value({'$mod': [d, m]})

    __eq__ = value

    __or__ = list_maker('$or')
    __and__ = list_maker('$and')

    nor = list_maker('$nor')

    __ne__ = operand_maker('$ne')
    __gt__ = operand_maker('$gt')
    __lt__ = operand_maker('$lt')
    __ge__ = operand_maker('$gte')
    __le__ = operand_maker('$lte')

    exists = operand_maker('$exists')
    contained = operand_maker('$in')
    notcontained = operand_maker('$nin')
    regex = operand_maker('$regex')
    options = operand_maker('$options')
    all = operand_maker('$all')
    type = operand_maker('$type')
    where = operand_maker('$where')
    size = operand_maker('$size')
    elemMatch = operand_maker('$elemMatch')

del operand_maker, list_maker

Q = Query()




import functools
import greenlet
from tornado.concurrent import Future

def asynchronize(loop):

    def sub_asynchronize(sync_method):
        
        @functools.wraps(sync_method)
        def method(*args, **kwargs):
            callback = kwargs.pop('callback', None)

            if callback:
                if not callable(callback):
                    raise callback_type_error
                future = None
            else:
                future = Future()

            def call_method():
                # Runs on child greenlet
                # TODO: ew, performance?
                try:
                    result = sync_method(*args, **kwargs)
                    if callback:
                        # Schedule callback(result, None) on main greenlet
                        loop.add_callback(functools.partial(
                            callback, result))
                    else:
                        # Schedule future to be resolved on main greenlet
                        loop.add_callback(functools.partial(
                            future.set_result, result))
                except Exception, e:
                    if callback:
                        loop.add_callback(functools.partial(
                            callback, e))
                    else:
                        loop.add_callback(functools.partial(
                            future.set_exception, e))

            # Start running the operation on a greenlet
            greenlet.greenlet(call_method).switch()
            return future

        return method
    return sub_asynchronize


__all__ = [
    'Field',
    'ForeignKey',
    'NumberField',
    'StringField',
    'ObjectField',
    'DateTimeField',
    'ListField'
]

from datetime import datetime
import re

class Field(object):

    def __init__(self, required=False, default=None, alias='', extra=lambda x:True):
        self.alias = alias
        self.extra = extra
        self.required = required
        self.default = default

    def init(self, obj):
        obj[self.key] = self.default() if callable(self.default) else self.default

    def __get__(self, obj, owner):
        return obj.get(self.key, self.default)

    def __set__(self, obj, value):
        obj[self.key] = value

    def __delete__(self, obj):
        pass

    def set_key(self, key):
        self.key = self.alias or key

    def validate(self, obj):
        value = obj.get(self.key, None)

        if self.required and not value:
            return False

        return self.extra(value)


class OneToManyRelation(object):

    def __init__(self, connected_class, key):
        self.connected_class = connected_class
        self.key = key

    def __get__(self, obj, owner):
        return self.connected_class.Q.filter({self.key:obj['_id']}).fetch()

    def __set__(self, obj, value):
        pass

    def __delete__(self, obj):
        pass


class ForeignKey(Field):

    def __init__(self, related_model, relation_name='', **kwargs):
        super(ForeignKey, self).__init__(**kwargs)

        self.related_model = related_model
        self.relation_name = relation_name

    def __get__(self, obj, owner):
        rel_id = obj.get(self.key, 0)
        if rel_id:
            return self.related_model.Q.get(rel_id)

    def __set__(self, obj, value):
        if hasattr(value, 'id'):
            obj[self.key] = value.id

    def __delete__(self, obj):
        pass

    def install_relation(self, owner_class, class_name):
        relation_name = self.relation_name or ('%s_list' % class_name.lower())
        setattr(self.related_model, relation_name, OneToManyRelation(owner_class, self.key))


class NumberField(Field):

    def __init__(self, default=0, type=int, precision=0, constraints=(None, None), **kwargs):
        super(NumberField, self).__init__(**kwargs)

        self.type = type
        self.precision = precision
        self.constraints = constraints
        self.default = default

    def __get__(self, obj, owner):
        value = super(NumberField, self).__get__(obj, owner)

        if self.precision:
            value = round(value, self.precision)

        return self.type(value)

    def __set__(self, obj, value):
        obj[self.key] = self.type(value)

    def validate(self, obj):
        if super(NumberField, self).validate(obj):
            value = obj[self.key]
            _min, _max = self.constraints
            return ((_min == None) or (value >= _min)) and ((_max == None) or (value <= _max))
        else:
            return False


class StringField(Field):

    def __init__(self, pattern='', len_constraints=(None, None), **kwargs):
        super(StringField, self).__init__(**kwargs)

        self.pattern = re.compile(pattern)
        self.constraints = len_constraints

    def validate(self, obj):
        if super(StringField, self).validate(obj):
            value = obj[self.key]
            print 'pre RE match'
            print self.pattern.match(value)
            if self.pattern.match(value) is None:
                return False
            print 'RE match'
            _min, _max = self.constraints
            return ((_min == None) or (len(value) >= _min)) and ((_max == None) or (len(value) <= _max))
        else:
            return False


class ObjectField(Field):

    def __init__(self, default={}, **kwargs):
        super(ObjectField, self).__init__(**kwargs)

        self.default = dict(default)


class ListField(Field):

    def __init__(self, default=[], **kwargs):
        super(ObjectField, self).__init__(**kwargs)

        self.default = list(default)


class DateTimeField(Field):

    def __init__(self, default=datetime.now, constraints=(None, None), **kwargs):
        super(DateTimeField, self).__init__(**kwargs)

        self.constraints = constraints
        self.default = default

    def __set__(self, obj, value):
        if isinstance(value, int):
            obj[self.key] = datetime.fromtimestamp(value)
        elif isinstance(value, str):
            v = None
            for frmt in [
                    '%Y-%m-%dT%H:%M:%S.%f', # ISO 8601
                    '%Y-%m-%d %H:%M:%S.%f',
                    '%Y-%m-%dT%H:%M:%S',    # ISO 8601
                    '%Y-%m-%d %H:%M:%S',
                    '%Y-%m-%d'
                ]:
                try:
                    v = datetime.strptime(value, frmt)
                except:
                    pass

            if not v:
                raise Exception('Unknown datetime format DateTimeField.set()')

            obj[self.key] = v

        elif isinstance(value, datetime) or value is None:
            obj[self.key] = value
        else:
            raise Exception('Invalid type DateTimeField.set()')

    def validate(self, obj):
        if super(DateTimeField, self).validate(obj):
            value = obj[self.key]
            _min, _max = self.constraints
            return ((_min == None) or (value >= _min)) and ((_max == None) or (value <= _max))
        else:
            return False


